//raul taranu

class Board
{
	public static final int NUM_ROWS = 9;
	public static final int NUM_COLS = 9;
	public static final int MAX_SUCC = 300;
	
	//constructor
	public Board()
	{
		board = new Position[NUM_COLS][NUM_ROWS];		
		for (int col = 1; col < NUM_COLS; col++)
			for (int row = 1; row < NUM_ROWS; row++)
				board[col][row] = new Position(col, row);
		numPieces = 0;
		numMoves = 0;
		setup();
		toMove = Piece.WHITE;
		whiteKingCastled = blackKingCastled = false;
		whiteKingMoved = blackKingMoved = false;
		whiteRookAMoved = whiteRookHMoved = false;
		blackRookAMoved = blackRookHMoved = false;
		updateSuccessors();
	}
	
	//copy constructor
	public Board(Board aBoard)
	{
		this.board = new Position[NUM_COLS][NUM_ROWS];		
		for (int col = 1; col < NUM_COLS; col++)
			for (int row = 1; row < NUM_ROWS; row++)
				this.board[col][row] = new Position(aBoard.getPosition(col, row));
		this.blackPositions = aBoard.getBlackPositions();
		this.whitePositions = aBoard.getWhitePositions();
		this.numPieces = aBoard.getNumPieces();
		this.toMove = aBoard.nextMove();
		this.whiteKingCastled = aBoard.whiteKingCastled;
		this.blackKingCastled = aBoard.blackKingCastled;
		this.whiteKingMoved = aBoard.whiteKingMoved;
		this.blackKingMoved = aBoard.blackKingMoved;
		this.whiteRookAMoved = aBoard.whiteRookAMoved;
		this.whiteRookHMoved = aBoard.whiteRookHMoved;
		this.blackRookAMoved = aBoard.blackRookAMoved;
		this.blackRookHMoved = aBoard.blackRookHMoved;
		this.successors = aBoard.getSuccessors();
		this.numMoves = aBoard.getNumMoves();
	}
	
	//accessors
	public PositionsList getWhitePositions() 
	{ 
		return whitePositions; 
	}
	public PositionsList getBlackPositions() 
	{ 
		return blackPositions; 
	}
	public int nextMove () { return toMove; }
	public SuccessorsList getSuccessors() { return successors; }
	public Position getPosition(int col, int row) { return board[col][row]; }
	public int getNumPieces() { return numPieces; }
	public int getNumMoves() { return numMoves; }
	public Move getLastMove() { return lastMove; }
	public Piece getPiece(Position position)
	{
		return board[position.getCol()][position.getRow()].getPiece();
	}
	public boolean castled(int color)
	{
		if (color == Piece.WHITE)
			return whiteKingCastled;
		else
			return blackKingCastled;
	}
	public int opponent(int color)
	{
		if (color == Piece.WHITE)
			return Piece.BLACK;
		else
			return Piece.WHITE;
	}
	public PositionsList getPositions(int color)
	{
		updatePiecePositions();
		if (color == Piece.WHITE)
			return getWhitePositions();
		else
			return getBlackPositions();
	}
	public boolean stalemate()
	{
		PositionsList positions;
		Position currentPosition;
		Piece currentPiece;
		positions = getPositions(nextMove());
		for (int aPos = 0; aPos < positions.getNumPositions(); aPos++)
		{
			currentPosition = positions.getPosition(aPos);
			currentPiece = currentPosition.getPiece();
			if (!(currentPiece.getCaptures(this, currentPosition).isEmpty() &&
	            currentPiece.getMoves(this, currentPosition).isEmpty()))
				return false;				
		}
		return true;
	}
	public boolean isEmpty(Position position)
	{
		return board[position.getCol()][position.getRow()].isEmpty();
	}
	public Position getKingPos(int color)
	{
		Position kingNotFound = new Position(9, 9);
		PositionsList positions = getPositions(color);
		for (int aPos = 0; aPos < positions.getNumPositions(); aPos++)
		{
			Piece currentPiece = positions.getPosition(aPos).getPiece();			
			if (currentPiece.getRank() == Piece.KING)
			{
				kingNotFound = null;
				return positions.getPosition(aPos);				
			}
		}
		//System.exit(0);
		//System.out.println("There is a serious problem, the king could not be found!!!");
		return kingNotFound;
	}
	public boolean canBlockCheck(Position kingPos, Position attackerPos)
	{
		int color = kingPos.getPiece().getColor();
		King kingPiece = new King(color);
		PositionsList team, moves;
		Position currentPos, currentMove;
		Board copy;
		team = getPositions(color);
		
		if (attackerPos.getPiece().getRank() == Piece.KNIGHT || attackerPos.getPiece().getRank() == Piece.PAWN)
		{
			kingPiece = null;
			return false;
		}
		else
		{
			for (int aPos = 0; aPos < team.getNumPositions(); aPos++)
			{
				currentPos = team.getPosition(aPos);
				if (currentPos.getPiece().getRank() != Piece.KING)
				{
					moves = currentPos.getPiece().getMoves(this, currentPos);
					for (int aMove = 0; aMove < moves.getNumPositions(); aMove++)
					{
						currentMove = moves.getPosition(aMove);
						copy = new Board(this);
						copy.movePiece(currentPos, currentMove);
						if (!kingPiece.inCheck(copy, kingPos))
						{
							kingPiece = null;
							return true;
						}
					}
				}
			}
		}
		return false;
	}
	public boolean noLoser()
	{
		King king = new King(nextMove());

		if (king.checkmate(this, getKingPos(nextMove())))
		{
			king = null;
			return false;
		}
		else
		{
			king = null;
			return true;
		}
	}
	public int oldUtility()
	{		
		King king = new King(opponent(nextMove()));
		Position myKingPos = this.getKingPos(nextMove());
		int util = 0;
		if (king.inCheck(this, this.getKingPos(opponent(nextMove()))))
			util += 2;
		king = new King(nextMove());
		if (king.inCheck(this, myKingPos))
			util -= 2;
		if (king.checkmate(this, myKingPos))
			util -= 1000;
		king = null;
		PositionsList myPositions = getPositions(nextMove());
		PositionsList opponentPositions = getPositions(opponent(nextMove()));
		for (int aPos = 0; aPos < myPositions.getNumPositions(); aPos++)
			util += myPositions.getPosition(aPos).getPiece().getRank();
		for (int aPos = 0; aPos < opponentPositions.getNumPositions(); aPos++)
			util -= opponentPositions.getPosition(aPos).getPiece().getRank();
		return util;
	}
	private int utilityHelper(PositionsList positions, int color)
	{
		int util = 0;
		int currentRank, currentDefRank, numMoves;
		PositionsList defenders, moves, caps;
		Piece currentPiece, currentDefPiece;
		Position currentPosition, currentDefPosition;
		boolean nonPawnDefender = false;
		for (int aPos = 0; aPos < positions.getNumPositions(); aPos++)
		{
			currentPosition = positions.getPosition(aPos);
			currentPiece = currentPosition.getPiece();
			currentRank = currentPiece.getRank();
			switch (currentRank)
			{
				case Piece.KING:
					if (castled(color))
						util += 3;
					else if (canCastle(color, Position.QUEENSIDE) || canCastle(color, Position.KINGSIDE))
						util += 2;
					else if (!kingMoved(color) && !(rookMoved(color, Position.A) && rookMoved(color, Position.H)))
						util += 1;
					break;
				case Piece.QUEEN:
					util += 100;
					moves = currentPiece.getMoves(this, currentPosition);
					caps = currentPiece.getCaptures(this, currentPosition);
					if (!caps.isEmpty())
						util += 10;
					numMoves = moves.getNumPositions() + caps.getNumPositions();
					util = util + (10 * (int)Math.floor(Math.sqrt((double)numMoves) + 0.5));
					break;
				case Piece.ROOK:
					util += 50;
					defenders = currentPosition.getAttackers(this, opponent(color));
					if (defenders.getNumPositions() > 1)
						util += 20;
					else if (defenders.getNumPositions() == 1)
						util += 10;
					moves = currentPiece.getMoves(this, currentPosition);
					caps = currentPiece.getCaptures(this, currentPosition);
					if (!caps.isEmpty())
						util += 10;
					numMoves = moves.getNumPositions() + caps.getNumPositions();
					util = util + (10 * (int)Math.floor(Math.sqrt((double)numMoves) + 0.5));
					break;
				case Piece.BISHOP:
					util += 35;
					defenders = currentPosition.getAttackers(this, opponent(color));
					if (defenders.getNumPositions() > 1)
						util += 20;
					else if (defenders.getNumPositions() == 1)
						util += 10;
					moves = currentPiece.getMoves(this, currentPosition);
					caps = currentPiece.getCaptures(this, currentPosition);
					if (!caps.isEmpty())
						util += 10;
					numMoves = moves.getNumPositions() + caps.getNumPositions();
					util = util + (10 * (int)Math.floor(Math.sqrt((double)numMoves) + 0.5));
					break;
				case Piece.KNIGHT:
					util += 30;
					defenders = currentPosition.getAttackers(this, opponent(color));
					if (defenders.getNumPositions() > 1)
						util += 20;
					else if (defenders.getNumPositions() == 1)
						util += 10;
					moves = currentPiece.getMoves(this, currentPosition);
					caps = currentPiece.getCaptures(this, currentPosition);
					if (!caps.isEmpty())
						util += 10;
					numMoves = moves.getNumPositions() + caps.getNumPositions();
					util = util + (10 * (int)Math.floor(Math.sqrt((double)numMoves) + 0.5));
					break;
				case Piece.PAWN:
					util += 10;
					util = util + (2 * rowsAdvanced(currentPosition.getRow(), color));
					defenders = currentPosition.getAttackers(this, opponent(color));
					if (defenders.isEmpty())
						util -= 3;
					for (int aDef = 0; (aDef < defenders.getNumPositions() && !nonPawnDefender); aDef++)
					{
						currentDefPosition = defenders.getPosition(aDef);
						currentDefPiece = currentDefPosition.getPiece();
						currentDefRank = currentDefPiece.getRank();
						if (currentDefRank != Piece.PAWN)
							nonPawnDefender = true;
					}
					if (nonPawnDefender)
						util += 3;
					break;				
			}
		}
		return util;
	}
	public int utility()
	{
		int util = 0;
		int color = nextMove();
		Position myKingPos = getKingPos(nextMove());
		PositionsList myPositions = getPositions(color);
		PositionsList opponentPositions = getPositions(opponent(color));
		
		King oppKing = new King(opponent(nextMove()));
		King king = new King(nextMove());
		if (oppKing.inCheck(this, getKingPos(opponent(nextMove()))))
			util += 5;
		if (king.inCheck(this, myKingPos))
			util -= 5;
		if (king.checkmate(this, myKingPos))
			util -= 1000;
		
		util += utilityHelper(myPositions, color);
		util -= utilityHelper(opponentPositions, opponent(color));
		
		return util;
	}
	private int rowsAdvanced(int row, int color)
	{
		if (color == Piece.WHITE)
			return row - 2;
		else
			return 7 - row;
	}
	
	//output
	public void display()
	{
		for (int row = 8; row > 0; row--)
		{
			for (int col = 1; col < NUM_COLS; col++)
				board[col][row].display();
			System.out.print("\n");
		}
	} 
	
	//mutators
	public void decNumMoves() { numMoves--; }
	public void incNextMove()
	{
		toMove = opponent(toMove);
	}
	public void updateSuccessors()
	{
		//System.out.println("NEW CALL TO US");
		Board copy;
		Position currentPos, currentMove, currentCapture;
		Piece currentPiece;
		PositionsList currentMoves;
		CapturesList currentCaptures;
		successors = null;
		successors = new SuccessorsList(MAX_SUCC);
		PositionsList positions = getPositions(nextMove());
		for (int aPos = 0; aPos < positions.getNumPositions(); aPos++)
		{
			currentPos = positions.getPosition(aPos);
			currentPiece = currentPos.getPiece();
			currentMoves = currentPiece.getMoves(this, currentPos);
			currentCaptures = currentPiece.getCaptures(this, currentPos);
			for (int aMove = 0; aMove < currentMoves.getNumPositions(); aMove++)
			{
				currentMove = currentMoves.getPosition(aMove);
				copy = new Board(this);
//				currentPos.display();
//				currentMove.display();
//				System.out.print("\n");
				copy.movePiece(currentPos, currentMove);
				successors.addSuccessor(copy, currentPos, currentMove);
				copy = null;
			}
			for (int aCap = 0; aCap < currentCaptures.getNumPositions(); aCap++)
			{
				currentCapture = currentCaptures.getPosition(aCap);
				copy = new Board(this);
				copy.movePiece(currentPos, currentCapture);
				successors.addSuccessor(copy, currentPos, currentCapture);
				copy = null;
			}
		}
	}
	private void updatePiecePositions()
	{
		whitePositions = blackPositions = null;
		whitePositions = new PositionsList(16);
		blackPositions = new PositionsList(16);
		for (int col = 1; col < NUM_COLS; col++)
		{
			for (int row = 1; row < NUM_ROWS; row++)
			{
				if (board[col][row].getPiece() != null)
				{
					if (row == 8 && board[col][row].getPiece().getColor() == Piece.WHITE &&
						board[col][row].getPiece().getRank() == Piece.PAWN)
					{
						removePiece(board[col][row]);
						insertPiece(new Queen(Piece.WHITE), board[col][row]);
					}
					if (row == 1 && board[col][row].getPiece().getColor() == Piece.BLACK &&
						board[col][row].getPiece().getRank() == Piece.PAWN)
					{
						removePiece(board[col][row]);
						insertPiece(new Queen(Piece.BLACK), board[col][row]);
					}
						
					if (board[col][row].getPiece().getColor() == Piece.WHITE)
						whitePositions.addPosition(board[col][row]);
					else
						blackPositions.addPosition(board[col][row]);
				}
			}
		}
	}
	
	private void setup()
	{
		for (int col = 1; col <= 8; col++)
		{
			board[col][2].insertPiece(new Pawn(Piece.WHITE));
			board[col][7].insertPiece(new Pawn(Piece.BLACK));
		}
		board[Position.A][1].insertPiece(new Rook(Piece.WHITE));
		board[Position.H][1].insertPiece(new Rook(Piece.WHITE));
		board[Position.A][8].insertPiece(new Rook(Piece.BLACK));
		board[Position.H][8].insertPiece(new Rook(Piece.BLACK));
		board[Position.B][1].insertPiece(new Knight(Piece.WHITE));
		board[Position.G][1].insertPiece(new Knight(Piece.WHITE));
		board[Position.B][8].insertPiece(new Knight(Piece.BLACK));
		board[Position.G][8].insertPiece(new Knight(Piece.BLACK));
		board[Position.C][1].insertPiece(new Bishop(Piece.WHITE));
		board[Position.F][1].insertPiece(new Bishop(Piece.WHITE));
		board[Position.C][8].insertPiece(new Bishop(Piece.BLACK));
		board[Position.F][8].insertPiece(new Bishop(Piece.BLACK));
		board[Position.D][1].insertPiece(new Queen(Piece.WHITE));
		board[Position.D][8].insertPiece(new Queen(Piece.BLACK));
		board[Position.E][1].insertPiece(new King(Piece.WHITE));
		board[Position.E][8].insertPiece(new King(Piece.BLACK));
	}
	
	public void movePiece(Move move)
	{
		movePiece(move.getFrom(), move.getTo());
	}
	
	public void movePiece(Position from, Position to)
	{
		/* if the piece being moved is on A1 */
    	if (from.getCol() == Position.A && from.getRow() == 1)
			movedRook(Piece.WHITE, Position.A);
    	/* if the piece being moved is on A8 */
		else if (from.getCol() == Position.A && from.getRow() == 8)
			movedRook(Piece.BLACK, Position.A);
		/* if the piece being moved is on H1 */
		else if (from.getCol() == Position.H && from.getRow() == 1)
			movedRook(Piece.WHITE, Position.H);
		/* if the piece being moved is on H8 */
		else if (from.getCol() == Position.H && from.getRow() == 8)
			movedRook(Piece.BLACK, Position.H);
		/* if the piece being moved is on E1 */
		else if (from.getCol() == Position.E && from.getRow() == 1)
			movedKing(Piece.WHITE);
		/* if the piece being moved is on E8 */
		else if (from.getCol() == Position.E && from.getRow() == 8)
			movedKing(Piece.BLACK);
		
		/* if moving a King from its starting position (column E) */
		if (from.getCol() == Position.E && getPiece(from).getRank() == Piece.KING)
		{
			/* to column G */
			if (to.getCol() == Position.G)
			{
				/* this is a kingside castle, so move the rook too and make sure it is the correct player's turn next */
				movePiece(new Position(Position.H, from.getRow()), new Position(Position.F, from.getRow()));
				incNextMove();
				decNumMoves();
				castledKing(from.getPiece().getColor());
			}			
			/* or to column C */
			else if (to.getCol() == Position.C)
			{
				/* this is a queenside castle, so move the rook too and make sure it is the correct player's turn next */
				movePiece(new Position(Position.A, from.getRow()), new Position(Position.D, from.getRow()));
				incNextMove();
				decNumMoves();
				castledKing(from.getPiece().getColor());
			}
		}
		
		if (getPiece(from).getRank() == Piece.PAWN && lastMove != null)
		{
			if (from.getCol() != to.getCol() && isEmpty(to))
			{
				numPieces--;
				removePiece(lastMove.getTo());
			}
		}
		
		if (!isEmpty(to) && !from.equals(to))
			numPieces--;
		board[to.getCol()][to.getRow()].insertPiece(board[from.getCol()][from.getRow()].getPiece());
		if (!from.equals(to))
		{
			board[from.getCol()][from.getRow()].removePiece();
		}
		toMove = opponent(nextMove());
		numMoves++;
		lastMove = new Move(from, to);
		updatePiecePositions();
	}
			
	public void insertPiece(Piece aPiece, Position position)
	{
		board[position.getCol()][position.getRow()].insertPiece(aPiece);
		numPieces++;
	}
	
	public void removePiece(Position position)
	{
		board[position.getCol()][position.getRow()].removePiece();
		numPieces--;
	}
	
	private void castledKing(int color)
	{
		if (color == Piece.WHITE)
			whiteKingCastled = true;
		else
			blackKingCastled = true;
	}
	
	private void movedKing(int color)
	{
		if (color == Piece.WHITE)
			whiteKingMoved = true;
		else
			blackKingMoved = true;
	}
	
	private void movedRook(int color, int col)
	{
		if (color == Piece.WHITE)
		{
			if (col == Position.A)
				whiteRookAMoved = true;
			else 
				whiteRookHMoved = true;
		}
		else
		{
			if (col == Position.A)
				blackRookAMoved = true;
			else
				blackRookHMoved = true;
		}
	}
	
	private boolean kingMoved(int color)
	{
		if (color == Piece.WHITE)
			return whiteKingMoved;
		else
			return blackKingMoved;
	}
	
	private boolean rookMoved(int color, int col)
	{
		if (color == Piece.WHITE)
		{
			if (col == Position.A)
				return whiteRookAMoved;
			else 
				return whiteRookHMoved;
		}
		else
		{
			if (col == Position.A)
				return blackRookAMoved;
			else
				return blackRookHMoved;
		}
	}
	
	public boolean canEnPassant(Position pos)
	{
		if (lastMove != null && lastMove.pawnForwardTwo(this))
		{
			if (pos.getRow() == lastMove.getTo().getRow() && 
					(pos.getCol() - 1 == lastMove.getTo().getCol() || pos.getCol() + 1 == lastMove.getTo().getCol()))
				return true;
		}
		return false;
	}
	
	public boolean canCastle(int color, int side)
	{
		if (color == Piece.WHITE)
		{
			if (whiteKingMoved)
				return false;
			else
			{
				King wKing = new King(Piece.WHITE);
				if (wKing.inCheck(this, getKingPos(Piece.WHITE)))
					return false;
				wKing = null;
			}
			if (side == Position.KINGSIDE)
			{
				if (whiteRookHMoved)
					return false;
				else if (!getPosition(Position.F, 1).isEmpty())
					return false;
				else if (getPosition(Position.F, 1).isAttacked(this, Piece.WHITE))
					return false;
				else if (!getPosition(Position.G, 1).isEmpty())
					return false;
				else if (getPosition(Position.G, 1).isAttacked(this, Piece.WHITE))
					return false;					
			}
			else
			{
				if (whiteRookAMoved)
					return false;
				else if (!getPosition(Position.D, 1).isEmpty())
					return false;
				else if (getPosition(Position.D, 1).isAttacked(this, Piece.WHITE))
					return false;
				else if (!getPosition(Position.C, 1).isEmpty())
					return false;
				else if (getPosition(Position.C, 1).isAttacked(this, Piece.WHITE))
					return false;
				else if (!getPosition(Position.B, 1).isEmpty())
					return false;
				else if (getPosition(Position.B, 1).isAttacked(this, Piece.WHITE))
					return false;
			}
		}
		else
		{
			if (blackKingMoved)
				return false;
			else
			{
				King bKing = new King(Piece.BLACK);
				if (bKing.inCheck(this, getKingPos(Piece.BLACK)))
					return false;
				bKing = null;
			}
			if (side == Position.KINGSIDE)
			{
				if (blackRookHMoved)
					return false;
				else if (!getPosition(Position.F, 8).isEmpty())
					return false;
				else if (getPosition(Position.F, 8).isAttacked(this, Piece.BLACK))
					return false;
				else if (!getPosition(Position.G, 8).isEmpty())
					return false;
				else if (getPosition(Position.G, 8).isAttacked(this, Piece.BLACK))
					return false;
			}
			else
			{
				if (blackRookAMoved)
					return false;
				else if (!getPosition(Position.D, 8).isEmpty())
					return false;
				else if (getPosition(Position.D, 8).isAttacked(this, Piece.BLACK))
					return false;
				else if (!getPosition(Position.C, 8).isEmpty())
					return false;
				else if (getPosition(Position.C, 8).isAttacked(this, Piece.BLACK))
					return false;
				else if (!getPosition(Position.B, 8).isEmpty())
					return false;
				else if (getPosition(Position.B, 8).isAttacked(this, Piece.BLACK))
					return false;
			}
		}
		return true;
	}
		
	//instance variables
	private Position [][] board;
	private int numPieces;
	private int numMoves;
	private SuccessorsList successors;
	private PositionsList whitePositions;
	private PositionsList blackPositions;
	private int toMove;
	private Move lastMove;
	private boolean whiteKingMoved, blackKingMoved, whiteKingCastled, blackKingCastled;
	private boolean whiteRookAMoved, whiteRookHMoved, blackRookAMoved, blackRookHMoved;
}