// raul taranu

class MiniMaxPlayer extends Player
{
	public static final int MAX_POSSIBLE = 24;
	public static final int INFINITY = 1000000;

	//constructor
	public MiniMaxPlayer(String name)
	{
		super(name);
		ruyLopez = new MovesList(10);
		setupRuyLopez();
		inRuyLopezOpening = true;
	}	
	
	private void setupRuyLopez()
	{
		ruyLopez.addMove(new Move(new Position(Position.E, 2), new Position(Position.E, 4)));
		ruyLopez.addMove(new Move(new Position(Position.E, 7), new Position(Position.E, 5)));
		ruyLopez.addMove(new Move(new Position(Position.G, 1), new Position(Position.F, 3)));
		ruyLopez.addMove(new Move(new Position(Position.B, 8), new Position(Position.C, 6)));
		ruyLopez.addMove(new Move(new Position(Position.F, 1), new Position(Position.B, 5)));
	}

	public Board makeMove(Board board, Chess gm)
	{
		Position from, to;
		if (board.getNumMoves() > 4)
		{
			inRuyLopezOpening = false;
			ruyLopez = null;
		}
		if (inRuyLopezOpening)
		{
			Move move = ruyLopez.getMove(board.getNumMoves());
			from = move.getFrom();
			to = move.getTo();
			board.movePiece(move.getFrom(), move.getTo());
		}
		else
		{
			board.updateSuccessors();
			Move move = bestMove(board);
			from = move.getFrom();
			to = move.getTo();
			board.movePiece(move);
		}	
		
		return board;
	}
	
	public Move bestMove(Board board)
	{
		MiniMaxSearch search = new MiniMaxSearch(2);
		int bestValue = -INFINITY;
		/* Initialize PBV and Best Move to NULL */
		int PBV = -INFINITY;
	    Move bestMove;
	    /* Generate all possible moves for Player in State (M1, ..., Mb) */
	    MovesList moves = new MovesList(Board.MAX_SUCC);
	    SuccessorsList successors = board.getSuccessors();
	    for (int aSucc = 0; aSucc < successors.getNumSuccessors(); aSucc++)
	    {
	    	Successor currentSucc = successors.getSuccessor(aSucc);
	    	moves.addMove(new Move(currentSucc.getFrom(), currentSucc.getTo()));
	    }
	    
	    Board copy;
	    copy = new Board(board);
	    copy.movePiece(moves.getMove(0));
	     
	    bestValue = search.minimax(copy, MiniMaxSearch.MIN, 1);
	    bestMove = moves.getMove(0);

	    for (int k = 1; k < moves.getNumMoves(); k++)
	    {
	    	copy = new Board(board);
		    copy.movePiece(moves.getMove(k));
		    PBV = search.minimax(copy, MiniMaxSearch.MIN, 1);
	        if (PBV > bestValue)
	        {
	        	bestValue = PBV;
	            bestMove = moves.getMove(k);
	        }
	    }
	    return bestMove;
	}

	//instance variables
	
	private MovesList ruyLopez;
	private boolean inRuyLopezOpening;
}