//raul taranu

public abstract class Piece
{
	public static final int WHITE = 0;
	public static final int BLACK = 1;
	public static final int PAWN = 1;
	public static final int KNIGHT = 3;
	public static final int BISHOP = 4;
	public static final int ROOK = 5;
	public static final int QUEEN = 9;
	public static final int KING = 100;
	public static final int MAX_CAPTURES = 8;
	public static final int MAX_MOVES = 27;
	
	//constructor
	public Piece(int color, int rank)
	{
		this.color = color;
		this.rank = rank;
	}

	//accessors
	public int getColor()
	{
		return color;
	}

	public int getRank()
	{
		return rank;
	}
	
	public PositionsList getMoves(Board board, Position startingPos)
	{
		PositionsList attacked = getAttackedPositions(board, startingPos);
		PositionsList moves = new PositionsList(MAX_MOVES);
		int color = board.getPiece(startingPos).getColor();
		Position currentPos;
		Board copy;
		King kingPiece = new King(color);
		Position kingPosition;
		
		currentPos = startingPos;
		
		/* for each position this piece can reach */
		for (int aPos = 0; aPos < attacked.getNumPositions(); aPos++)
		{
			currentPos = attacked.getPosition(aPos);
			copy = new Board(board);
			copy.movePiece(startingPos, currentPos);
			kingPosition = copy.getKingPos(color);
			if (board.isEmpty(currentPos) && !kingPiece.inCheck(copy, kingPosition))
				moves.addPosition(currentPos);
		}
		if (getRank() == KING)
		{
			if (board.canCastle(color, Position.KINGSIDE))
			{
				currentPos = new Position(startingPos.getCol() + 2, startingPos.getRow());
				moves.addPosition(currentPos);
			}
			if (board.canCastle(color, Position.QUEENSIDE))
			{
				currentPos = new Position(startingPos.getCol() - 2, startingPos.getRow());
				moves.addPosition(currentPos);
			}
		}
		
		return moves;
	}

	public CapturesList getCaptures(Board board, Position startingPos)
	{
		PositionsList attacked = getAttackedPositions(board, startingPos);
		CapturesList captures = new CapturesList(MAX_CAPTURES);
		int color = board.getPiece(startingPos).getColor();
		Position currentPos;
		Board copy;
		King kingPiece = new King(color);
		Position kingPosition;
		
		/* for each position this piece can reach */
		for (int aPos = 0; aPos < attacked.getNumPositions(); aPos++)
		{
			currentPos = attacked.getPosition(aPos);
			copy = new Board(board);
			copy.movePiece(startingPos, currentPos);
			/* if the current position is occupied by an enemy piece, and if capturing that piece does not put our king in check */
			kingPosition = copy.getKingPos(color);
			if (!board.isEmpty(currentPos) && board.getPiece(currentPos).getColor() != color && !kingPiece.inCheck(copy, kingPosition))
				captures.addPosition(currentPos, board.getPiece(currentPos));
		}
		
		if (getRank() == PAWN)
		{
			if (board.canEnPassant(startingPos))
			{
				int row;
				if (color == WHITE)
					row = startingPos.getRow() + 1;
				else 
					row = startingPos.getRow() - 1;
				int col = board.getLastMove().getTo().getCol();
				currentPos = new Position(col, row);
				copy = new Board(board);
				copy.movePiece(startingPos, currentPos);
				/* if the current position is occupied by an enemy piece, and if capturing that piece does not put our king in check */
				kingPosition = copy.getKingPos(color);
				if (!kingPiece.inCheck(copy, kingPosition))
					captures.addPosition(currentPos, board.getPiece(board.getLastMove().getTo()));
			}
		}
		
		return captures;
	}
	public abstract PositionsList getAttackedPositions(Board board,	Position startingPos);

	public abstract void display();

	//instance variables
	private int color;

	private int rank;
}