// raul taranu

import javax.swing.*;

class Position
{
	public static final int KINGSIDE = 0;
	public static final int QUEENSIDE = 1;
	public static final int A = 1;
	public static final int B = 2;
	public static final int C = 3;
	public static final int D = 4;
	public static final int E = 5;
	public static final int F = 6;
	public static final int G = 7;
	public static final int H = 8;
	
	//constructors
	public Position(int col, int row)
	{
		this.col = col;
		this.row = row;
		this.piece = null;

		if ((this.col + this.row) % 2 == 0)
			this.strSquareColor = "dark";
		else
			this.strSquareColor = "light";
		String filespec = "./images/" + strSquareColor + ".png";

		this.positionIcon = new ImageIcon(filespec);
	}
	public Position(String col, String row)
	{
		if (col != null && row != null)
		{
			if (col.equals("A"))
				this.col = A;
			else if (col.equals("B"))
				this.col = B;
			else if (col.equals("C"))
				this.col = C;
			else if (col.equals("D"))
				this.col = D;
			else if (col.equals("E"))
				this.col = E;
			else if (col.equals("F"))
				this.col = F;
			else if (col.equals("G"))
				this.col = G;
			else if (col.equals("H"))
				this.col = H;
			
			if (row.equals("1"))
				this.row = 1;
			else if (row.equals("2"))
				this.row = 2;
			else if (row.equals("3"))
				this.row = 3;
			else if (row.equals("4"))
				this.row = 4;
			else if (row.equals("5"))
				this.row = 5;
			else if (row.equals("6"))
				this.row = 6;
			else if (row.equals("7"))
				this.row = 7;
			else if	(row.equals("8"))
				this.row = 8;
		}
		else
		{
			System.out.println("Bad position construction!!!!!");
			this.col = 0;
			this.row = 0;
		}
		
		this.piece = null;

		if ((this.col + this.row) % 2 == 0)
			this.strSquareColor = "dark";
		else
			this.strSquareColor = "light";
		String filespec = "./images/" + strSquareColor + ".png";

		this.positionIcon = new ImageIcon(filespec);
	}

	//copy contructor
	public Position(Position aPosition)
	{
		this.col = aPosition.getCol();
		this.row = aPosition.getRow();
		this.piece = null;
		this.insertPiece(aPosition.getPiece());
	}

	//accessors
	public int getCol()	{ return col; }
	public int getRow()	{ return row; }
	public Piece getPiece()	{ return piece; }
	public ImageIcon getIcon() { return positionIcon; }
	public String getSquareColor() { return strSquareColor; }
	public boolean equals(Position position)
	{
		return (getCol() == position.getCol() && getRow() == position.getRow());
	}
	public boolean isEmpty() { return (getPiece() == null);	}
	public boolean isAttacked(Board board, int friendlyColor)
	{
		return !(getAttackers(board, friendlyColor).isEmpty());
	}
	public PositionsList getAttackers(Board board, int friendlyColor)
	{
		PositionsList attackers = new PositionsList(15);
		PositionsList positions = board.getPositions(board.opponent(friendlyColor));
				
		for (int aPos = 0; aPos < positions.getNumPositions(); aPos++)
		{
			Piece currentPiece = positions.getPosition(aPos).getPiece();
			
			if (currentPiece.getAttackedPositions(board, positions.getPosition(aPos)).member(this))
				attackers.addPosition(positions.getPosition(aPos));
		}
		return attackers;
	}

	//output
	public void display()
	{
		switch (getCol())
		{
			case A:
				System.out.print("A");
				break;
			case B:
				System.out.print("B");
				break;
			case C:
				System.out.print("C");
				break;
			case D:
				System.out.print("D");
				break;
			case E:
				System.out.print("E");
				break;
			case F:
				System.out.print("F");
				break;
			case G:
				System.out.print("G");
				break;
			case H:
				System.out.print("H");
				break;
		}

		System.out.print(getRow() + " : ");

		if (!isEmpty())
			getPiece().display();
		else
			System.out.print("             ");
	}

	//mutators
	public void insertPiece(Piece aPiece)
	{
		this.piece = aPiece;
		if ((this.col + this.row) % 2 == 0)
			this.strSquareColor = "dark";
		else
			this.strSquareColor = "light";

		String strRank;
		String strPieceColor;

		if (aPiece != null)
		{
			if (aPiece.getColor() == Piece.WHITE)
				strPieceColor = "_white_";
			else
				strPieceColor = "_black_";

			switch (aPiece.getRank())
			{
				case Piece.PAWN:
					strRank = "pawn";
					break;
				case Piece.KNIGHT:
					strRank = "knight";
					break;
				case Piece.BISHOP:
					strRank = "bishop";
					break;
				case Piece.ROOK:
					strRank = "rook";
					break;
				case Piece.QUEEN:
					strRank = "queen";
					break;
				default:
					strRank = "king";
					break;
			}
		}
		else
		{
			strPieceColor = "";
			strRank = "";
		}

		String filespec = "./images/" + strSquareColor + strPieceColor + strRank
				+ ".png";
		this.positionIcon = new ImageIcon(filespec);
	}

	public void removePiece()
	{
		this.piece = null;

		if ((this.col + this.row) % 2 == 0)
			this.strSquareColor = "dark";
		else
			this.strSquareColor = "light";

		String filespec = "./images/" + strSquareColor + ".png";
		this.positionIcon = new ImageIcon(filespec);
	}

	//instance variables
	private int row;

	private int col;

	private Piece piece;

	private ImageIcon positionIcon;

	private String strSquareColor;
}