// Raul Taranu

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

class Table extends JFrame implements ActionListener
{
	public Table(String name, Chess gm)
	{
		
		super(name);
		this.gm = gm;
		Container contentPane = getContentPane();
		Component contents = createComponents();
		contentPane.add(contents);
		addWindowListener(new WindowAdapter()
		{ // GUIWindow's close event handler
			public void windowClosing(WindowEvent we)
			{
				System.exit(0);
			}
		});

		pack();
		setVisible(true);
	}

	private Component createComponents()
	{
		label = new JLabel(labelPrefix + gm.getBoard().nextMove());
		infoPanel = new JPanel();
		infoPanel.setLayout(new BoxLayout(infoPanel, BoxLayout.Y_AXIS));
		infoPanel.add(label);

		JMenuItem menuItem;
		menuBar = new JMenuBar();
		setJMenuBar(menuBar);
		menu = new JMenu("Game");
		menu.setMnemonic(KeyEvent.VK_G);
		menuBar.add(menu);

		menuItem = new JMenuItem("Start New Game");
		menuItem.setActionCommand("new");
		menuItem.setMnemonic(KeyEvent.VK_N);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, ActionEvent.CTRL_MASK));
		menuItem.addActionListener(this);
		menu.add(menuItem);

		menuItem = new JMenuItem("Current Utility");
		menuItem.setActionCommand("utility");
		menuItem.setMnemonic(KeyEvent.VK_U);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_U, ActionEvent.CTRL_MASK));
		menuItem.addActionListener(this);
		menu.add(menuItem);

		boardPanel = new BoardPanel(gm);
		boardPanel.setLayout(new GridLayout(8, 8));
		boardPanel.addBoardButtons();

		JPanel window = new JPanel();
		window.setLayout(new BoxLayout(window, BoxLayout.X_AXIS));
		window.add(boardPanel);
		//window.add(infoPanel);
		return window;
	}

	public void dialog(String msg)
	{
		JOptionPane.showMessageDialog(this, msg, "FYI", JOptionPane.INFORMATION_MESSAGE);
	}

	public void update()
	{
		if (gm.getBoard().nextMove() == Piece.WHITE)
			label.setText(labelPrefix + "white.");
		else
			label.setText(labelPrefix + "black.");
		boardPanel.update();
	}

	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e)
	{
		JMenuItem source = (JMenuItem) (e.getSource());

		if (e.getActionCommand().equals("new"))
		{
			setVisible(false);
			gm.newGame();
		}
		else if (e.getActionCommand().equals("utility"))
		{
			String color;
			if (gm.getBoard().nextMove() == Piece.WHITE)
				color = "WHITE";
			else
				color = "BLACK";
			String msg = "The utility of the current board for " + color + " is: " + gm.getBoard().utility(); 
			dialog(msg);
		}
	}

	public String getHostIP() { return HostIP; }
	public Position getPosition()
	{
		return boardPanel.getPosition();
	}

	public Player getPlayer(String color)
	{
		Object[] possibilities = {"human", "novice", "Alpha-Beta", "MiniMax"};
		String s;
		do
		{
			s = (String) JOptionPane.showInputDialog(
					this,
					"Choose a player for " + color,
					"Choose Player", 
					JOptionPane.PLAIN_MESSAGE, 
					null,
					possibilities, 
					"human");
		}
		while (s == null);
		
		if(s.equals("human"))
			return new HumanPlayer(color);
		else if (s.equals("novice"))
			return new NovicePlayer(color);
		else if (s.equals("Alpha-Beta"))
			return new AlphaBetaPlayer(color);
		else
			return new MiniMaxPlayer(color);
	}
	
	public String getGameType()
	{
		Object[] possibilities = {"Local Game", "Network Game - Join", "Network Game - Host"};
		String s;
		do
		{
			s = (String) JOptionPane.showInputDialog(
					this,
					"What type of game to you want to play?",
					"Choose Game Type", 
					JOptionPane.QUESTION_MESSAGE, 
					null,
					possibilities, 
					"Local Game");
		}
		while (s == null);
		if (s.equals("Local Game"))
			return "local";
		else if (s.equals("Network Game - Join"))
		{
			HostIP = getIP();
			return "join";
		}
		else
			return "host";
	}
	
	private String getIP()
	{
		String IP;
		do
		{
			IP = (String) JOptionPane.showInputDialog(
					this,
                    "Enter the IP address or hostname of the game to connect to\n",
                    "IP Address",
                    JOptionPane.PLAIN_MESSAGE);
		}
		while (IP == null);
		
		return IP;
	}
	
	// instance variables

	private static String labelPrefix = "Next Turn: ";
	private Chess gm;
	private boolean pack = true;
	private Position position;
	private BoardPanel boardPanel;
	private JMenuBar menuBar;
	private JMenu menu;
	private JPanel infoPanel;
	private JLabel label;
	private String HostIP;
}